# -*- coding: utf-8 -*-
"""
@author: Ce Song from Dalian University of Technology

Functions for performing the feature selection procedure.
"""

import random
import pandas as pd

from sklearn.feature_selection import VarianceThreshold
from sklearn.preprocessing import StandardScaler
from sklearn.feature_selection import RFECV
from sklearn.model_selection import KFold
from sklearn.ensemble import RandomForestRegressor



# First step in feature selection procedure: remove the constant descriptors
def del_constant(df_train):
    column_names = df_train.columns.tolist()
    column_names.remove('Tg')
    column_names.remove('data_point')
    All_des = df_train[column_names].values
    selector = VarianceThreshold()
    selector.fit_transform(All_des)
    sel = selector.get_support(indices=True)
    
    # Get selected descriptors' names
    sel_name = []
    for i in range(len(sel)):
        m = sel[i]
        sel_name.append(column_names[m])
    
    cols = ['data_point'] + sel_name + ['Tg']
    df = df_train[cols]
    return df



# Second step in feature selection procedure: perform the recursive feature elimination algorithm with cross-validation (RFECV)
def rfecv_RF(train_df):
       
    X = train_df.drop(columns = ['data_point', 'Tg']).values
    Y = train_df['Tg'].values
    scaler = StandardScaler().fit(X)
    X_stand = scaler.transform(X)
    CV = KFold(n_splits = 5, random_state = 33)
    estimator = RandomForestRegressor(max_depth = 10, random_state = 33, n_estimators = 90)
    selector = RFECV(estimator, step = 1, scoring = "neg_mean_squared_error", cv = CV, n_jobs = 8)
    selector.fit(X_stand,Y)
    selCV_fea = selector.support_
    selCV_descriptors = []
    descriptors_name = train_df.drop(columns = ['data_point','Tg']).columns.tolist()
    for i in range(len(descriptors_name)):
        temp = descriptors_name[i]
        if selCV_fea[i] == True:
            selCV_descriptors.append(temp)
    
    cols = ['data_point'] + selCV_descriptors + ['Tg']
    df = train_df[cols]
    
    return df



# Third step in feature selection procedure: screen out highly colinear features
def del_Colinear(train_df):
    
    threshold = 0.95
    
    df = train_df.drop(columns = ['data_point'])
    r_pearson = df.corr(method = 'pearson')
    del_name = []
    for i in range(r_pearson.shape[0]-1):
        if r_pearson.columns[i] in del_name:
            continue
        else:
            for j in range(i+1, r_pearson.shape[0]-1):
                if abs(r_pearson.iat[i,j]) > threshold:
                    r_tg_i = r_pearson.loc[r_pearson.columns[i], 'Tg']
                    r_tg_j = r_pearson.loc[r_pearson.columns[j], 'Tg'] 
                    if abs(r_tg_i) >= abs(r_tg_j):
                        if r_pearson.columns[j] not in del_name:
                            del_name.append(r_pearson.columns[j])
                    else:
                        if r_pearson.columns[i] not in del_name:
                            del_name.append(r_pearson.columns[i])
                            break
                else:
                    continue
                
    newdf = train_df.drop(columns = del_name)
    
    return newdf




        
    
    