# -*- coding: utf-8 -*-
"""
@author: Ce Song from Dalian University of Technology

Script for generating the training and test sets for constructing machining learning models.
"""

import random
import pandas as pd
import os


# Read in dataset
df_ML = pd.read_csv('./dataset_ML.csv')
df_raw = pd.read_csv('./dataset_raw.csv')



# Parameters
rand_seed = 33831



def main():
    
    if not os.path.exists('./train_test_data'):
        os.mkdir('./train_test_data')
        
    train_df = train_test_split(df_ML, df_raw, rand_seed)[0]
    train_df.to_csv('./train_test_data/train_1.csv' , index = 0)
    test_df = train_test_split(df_ML, df_raw, rand_seed)[1]   
    test_df.to_csv('./train_test_data/test_1.csv' , index = 0)

    
    
def train_test_split(df_ML, df_raw, rand):
    count_range = [0, 50, 100, 150, 200, 250, 300, 350, 400]
    df_ML.set_index('data_point')
    df_raw.set_index('data_point')
    random.seed(rand)
    data_index = df_ML.index.to_list()
    OH_names = []
    X_names = []
    index_tr = []
    for i in data_index:
        for j in [1, 2, 3, 4]:
            OH = df_raw.loc[i,'a%s' % j]
            if type(OH) == float:
                continue
            elif type(OH) == str:
                if OH not in OH_names:
                    OH_names.append(OH)
                    index_tr.append(i)
                    for k in [1, 2, 3, 4]:
                        X = df_raw.loc[i, 'b%s' % k]
                        if type(X) == float:
                            continue
                        elif type(X) == str:
                            if X not in X_names:
                                X_names.append(X)
                                index_tr.append(i)
    
    index_train = list(set(index_tr))
    
    
    index_Train = []
    index_Test = []
    m = 0
    train_num = round(len(data_index)*0.8)
    if train_num < len(index_train):
        print('The current dataset cannot meet the requirement that all monomers exist in the training set which holds 80% data points of the dataset!')
        
        return 0
    else: 
        test_num = len(data_index) - train_num
        ratio = test_num/(len(data_index) - len(index_train))
        while m < len(count_range)-1:
            low = count_range[m]
            high = count_range[m+1]
            index_m = []
            for n in data_index:
                if n not in index_train:
                    if df_raw.loc[n, 'Tg'] > low and df_raw.loc[n, 'Tg'] <= high:
                        index_m.append(n)
            test_num_m = round(len(index_m)*ratio)           
            index_test_m = random.sample(index_m, test_num_m)
            index_Test += index_test_m
            for dele in index_test_m:
                index_m.remove(dele)
            index_train_m = index_m
            index_Train += index_train_m
            
            m += 1
        
        index_TRAIN = index_train + index_Train
        #print('Train: '+ str(len(index_TRAIN))+ ';  Test: '+ str(len(index_Test))+'. \n')
        df_train = pd.DataFrame(df_ML, index = index_TRAIN)
        df_train.reset_index()
        df_train.rename(columns = {'index':'data_point'})
        
        df_test = pd.DataFrame(df_ML, index = index_Test)
        df_test.reset_index()
        df_test.rename(columns = {'index':'data_point'})
        
        
        return df_train, df_test
        
if __name__ == '__main__':

    main()
    
    

    
    
    