# -*- coding: utf-8 -*-
"""
@author: Ce Song from Dalian University of Technology

Script for training machine learning models, evaluating the performance of trained models, and using the trained models to predict Tg of new PAEs.
"""

import pandas as pd
import numpy as np
import os
import pickle
from sklearn.svm import SVR
from sklearn.preprocessing import StandardScaler
from sklearn.metrics import mean_squared_error, r2_score
import Feature_selection_Funcs as FN


# Read in train/test data
train_df = pd.read_csv('./train_test_data/train_1.csv')
test_df = pd.read_csv('./train_test_data/test_1.csv')

# Read in validation data, i.e., new PAEs
valid_df = pd.read_csv('./validation_data_ML.csv')

# Parameters
c = 200  



def main():

    if not os.path.exists('./models'):
        os.mkdir('./models')

    # feature selection procedure
    ## remove constant features
    df_t1 = FN.del_constant(train_df)
    
    ## RFECV with Random Forest
    df_t2 = FN.rfecv_RF(df_t1)
    
    ## remove colinear features
    df = FN.del_Colinear(df_t2)
    
    
    # train model 
    selected_descriptors = df.drop(columns = ['data_point', 'Tg']).columns.tolist()
    X_train = df.drop(columns = ['data_point', 'Tg']).values
    X_test = test_df[selected_descriptors]
    Y_train = df['Tg']
    Y_test = test_df['Tg']
    scaler = StandardScaler().fit(X_train)
    X_train_std = scaler.transform(X_train)
    X_test_std = scaler.transform(X_test)
        
    SVR_estimator = SVR(kernel = 'rbf' ,C = c, gamma = 'scale')
    SVR_estimator.fit(X_train_std, Y_train)
    Y_pre_train = SVR_estimator.predict(X_train_std)
    Y_pre_test = SVR_estimator.predict(X_test_std)
    
    
    # evaluate model
    rmse_train = np.sqrt(mean_squared_error(Y_train, Y_pre_train))
    r2_train = r2_score(Y_train, Y_pre_train)
    rmse_test = np.sqrt(mean_squared_error(Y_test, Y_pre_test))
    r2_test = r2_score(Y_test, Y_pre_test)
    
    
    # save model
    filename = './models/model_1.pkl' 
    pickle.dump(SVR_estimator, open(filename, 'wb'))
    
    
    # predict Tg of new PAEs
    X_valid = valid_df[selected_descriptors]
    X_valid_std = scaler.transform(X_valid)
    Y_pre_valid = SVR_estimator.predict(X_valid_std)
    
    
    # save results
    result_df = pd.DataFrame(columns = ['Features', 'RMSE_train', 'RMSE_test', 'R2_train', 'R2_test', 'valid'])
    result_df.loc[len(result_df.index)] = [selected_descriptors, rmse_train, rmse_test, r2_train, r2_test, Y_pre_valid]
    result_df.to_csv('./models/ML_results_1.csv')
    
    
    
    
    
  
            
            
    
if __name__ == '__main__':
    main()            
        
        
        
        
        
        
        
        
        
        
